import React, { Component } from 'react';
import { View } from 'react-native';
import { RNCamera } from 'react-native-camera';
import PropTypes from 'prop-types';
import getStyles from './BarcodeScanner.styles';

/**
 * This component is designed to allow an application to scan barcodes.
 * Scanning features are disabled by default. To enable set the scan property to true
 * @property { object } contextStyles - Styles for this component
 * @property { bool } scan - Tells the component to start/stop scanning.
 * @property { function( newState ) } onScanStateChanged - Informs the listner of scanning state change
 * @property { function( barcodeData ) } onBarcodeScanned - Informs the listner of scanned barcode data and disables scanning
 */
class BarcodeScan extends Component {

    static defaultProps = {
        contextStyles: {},
        scan: false,
        onScanStateChanged: (newState) => { return newState },
        onBarcodeScanned: (barcodeData) => { return barcodeData; }
    };

    static propTypes = {
        /** Styles for this component */
        contextStyles: PropTypes.shape({

            /** Styling for the main camera view container */
            container: PropTypes.object,

            /** Styles for the camera layout */
            cameraView: PropTypes.object,
        }),
        /** Tells the component to start scanning. */
        scan: PropTypes.bool,
        /** Informs the caller about scanning state */
        onScanStateChanged: PropTypes.func,
        /**
         * Called when barcode data has been scanned successfully.
         * @param { object } barcodeData - Object containing barcode information
         */
        onBarcodeScanned: PropTypes.func,
    };

    constructor(props) {
        super(props);
        this.scannerEnabled = props.scan;
    }

    render() {
        const { contextStyles } = this.props;
        const styles = getStyles(contextStyles);

        return (
            <View style={styles.container}>
                <RNCamera
                    style={styles.cameraView}
                    onBarCodeRead={this.handleOnBarCodeRead}
                    captureAudio={false}
                    ref={cam => this.camera = cam}
                >
                </RNCamera>
            </View>
        )
    }

    /**
     * Used to help manage prop transition.
     * @param {object} prevProps - previous props 
     */
    componentDidUpdate(prevProps) {
        const { scan: thisScan } = this.props;
        const { scan: prevScan } = prevProps;

        if (thisScan !== prevScan) {
            handleOnScanStateChanged(thisScan);
        }
    }

    /**
     * Notify listners of state change. This will let you know
     * what the current state of scanning is.
     * @param { bool } scanState - Bool of new state.
     */
    handleOnScanStateChanged = (scanState) => {

        // If we have listners notify them of state change.
        const { onScanStateChanged } = this.props;
        if (onScanStateChanged) {
            onScanStateChanged(scanState);
        }

        // Enable or disable scan
        this.scannerEnabled = scanState;
        return this.scannerEnabled;
    }

    /**
     * Called by RNCamera when the barcode has been translated.
     * @param { object } barcodeData - Captured by the camera.
     */
    handleOnBarCodeRead = (barcodeData) => {
        // If the scanner is enabled broadcast barcode data
        if (this.scannerEnabled) {
            // turn off scanner. This handler is very noisy so we want to disable as soon as possible.
            this.scannerEnabled = false;

            // Broadcast barcode data to listners
            const { onBarcodeScanned } = this.props;
            onBarcodeScanned(barcodeData);

            // Raise event that scan state has changed.
            this.handleOnScanStateChanged(this.scannerEnabled);

            // If in development mode, log to console barcode data.
            const isDevEnv = process.env.NODE_ENV === 'development';
            if (isDevEnv) {
                console.log(`Barcode Data: \r\n ${JSON.stringify(barcodeData)}`);
            }
        }
    }
}

export default BarcodeScan;
